.invisible { visibility: hidden; }
.hidden { display: none; }
ul.container {
    list-style-type: none;
    margin: 0em;
    padding: 0em;
}
li {
    margin: 0;
}
li.cell {
    float: left;
    overflow: hidden;
}
.cell:not(.dummy):not(.hidden) {
    margin-left: 0%;
}
.cell:not(.dummy):not(.hidden) ~ .cell:not(.dummy):not(.hidden) {
    margin-left: 1%;
}
.item.result-pass { background-color: #077; }
.item.result-fail { background-color: #f77; }
.item.result {
    padding: 3px;
    -moz-border-radius: 5px;
    -webkit-border-radius: 5px;
}
.item-inner.result {
    -moz-border-radius: 2px;
    -webkit-border-radius: 2px;
}

.item-inner {
  overflow:hidden;
  padding: 2px;
}
.item-inner.test { background-color: #ddd; }
.item-inner.build { background-color: #ccc; }
.item { clear: left; }
.dummy { width: 0; }
.container.days {
    font-size: smaller;
    font-family: ubuntu, sans;
}
.container.trees {
    margin-left: 8ex;
}
li.commit {
    margin-top: 3px;
}
.item.commit {
    background-color: #aaa;
    -moz-border-radius: 5px;
    -webkit-border-radius: 5px;
    padding: 2px;
}
.tree.cell {
    margin-top: 0.5em;
    margin-bottom: 1em;
}
.item.day {
    padding: 0.5em;
    background-color: silver;
}
.item.tree {
    font-weight: bold;
    text-align: center;
}
.pass-delta.more-passes, .pass-delta.fewer-passes {
    font-weight: bold;
}
.pass-delta {
    color: green;
}
.fail-delta.more-fails, .fail-delta.fewer-fails {
    font-weight: bold;
}
.fail-delta {
    color: red;
}
li.day {
    clear: left;
    margin-top: 1ex;
}